@@ -2,11 +2,12 @@ import functools
 import json
 import types
 import urllib.parse
-from typing import Dict, Generator, Optional, Union
+from typing import Dict, Generator, Optional, Sequence, Union
 
 from cognite.client._api.geospatial import GeospatialAPI
-from cognite.client.data_classes.geospatial import Feature
+from cognite.client.data_classes.geospatial import Feature, FeatureList
 from cognite.client.exceptions import CogniteConnectionError, CogniteException
+from cognite.client.utils._identifier import IdentifierSequence, SingletonIdentifierSequence
 from requests.exceptions import ChunkedEncodingError
 
 from cognite.experimental.data_classes.geospatial import *
@@ -287,7 +288,10 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
         """
         resource_path = ExperimentalGeospatialAPI._MVT_RESOURCE_PATH
         return self._create_multiple(
-            items=mappings_definitions, resource_path=resource_path, cls=MvpMappingsDefinitionList
+            items=mappings_definitions,
+            resource_path=resource_path,
+            list_cls=MvpMappingsDefinitionList,
+            resource_cls=MvpMappingsDefinition,
         )
 
     @_with_cognite_domain
@@ -310,7 +314,9 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
                 >>> res = c.geospatial.delete_mvt_mappings_definitions(external_id="surveys")
         """
         resource_path = ExperimentalGeospatialAPI._MVT_RESOURCE_PATH
-        return self._delete_multiple(external_ids=external_id, wrap_ids=True, resource_path=resource_path)
+        return self._delete_multiple(
+            IdentifierSequence.load(external_ids=external_id), wrap_ids=True, resource_path=resource_path
+        )
 
     @_with_cognite_domain
     def retrieve_mvt_mappings_definitions(self, external_id: Union[str, List[str]] = None) -> MvpMappingsDefinitionList:
@@ -333,8 +339,12 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
                 >>> c.geospatial.retrieve_mvt_mappings_definitions(external_id="surveys")
         """
         resource_path = ExperimentalGeospatialAPI._MVT_RESOURCE_PATH
+        identifiers = IdentifierSequence.load(external_ids=external_id)
         return self._retrieve_multiple(
-            wrap_ids=True, external_ids=external_id, resource_path=resource_path, cls=MvpMappingsDefinitionList
+            identifiers=identifiers.as_singleton() if identifiers.is_singleton() else identifiers,
+            resource_path=resource_path,
+            list_cls=MvpMappingsDefinitionList,
+            resource_cls=MvpMappingsDefinition,
         )
 
     @_with_cognite_domain
@@ -354,7 +364,12 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
                 >>> c.geospatial.list_mvt_mappings_definitions()
         """
         resource_path = ExperimentalGeospatialAPI._MVT_RESOURCE_PATH
-        return self._list(method="POST", cls=MvpMappingsDefinitionList, resource_path=resource_path)
+        return self._list(
+            method="POST",
+            list_cls=MvpMappingsDefinitionList,
+            resource_cls=MvpMappingsDefinition,
+            resource_path=resource_path,
+        )
 
     @_with_cognite_domain
     def compute(
@@ -362,6 +377,8 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
         sub_computes: Dict[str, Any] = None,
         from_feature_type: str = None,
         filter: Dict[str, Any] = None,
+        group_by: Sequence[Dict[str, Any]] = None,
+        order_by: Sequence[ComputeOrder] = None,
         output: Dict[str, Any] = None,
         binary_output: Dict[str, Any] = None,
     ) -> Union[bytes, ComputedItemList]:
@@ -372,6 +389,8 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
             sub_computes (Dict[str, Any]): the sub-computed data for the main compute
             from_feature_type (str): the main feature type external id to compute from
             filter (Dict[str, Any]): the filter for the main feature type
+            group_by (List[Dict[str, Any]]): the list of group by expressions
+            order_by (List[ComputeOrder]): the list of order by expressions and direction
             output (Dict[str, Any]): the output json spec
             binary_output (Dict[str, Any]): the binary output computation to execute
 
@@ -437,10 +456,35 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
                 ...         "from4326to102016": {"stTransform": {"geometry": {"ref": "paris"}, "srid": 102016}},
                 ...     }
                 ... )
+
+            Compute the sorted aggregation result from a feature type
+
+                >>> client.geospatial.compute(
+                ...     from_feature_type="someFeatureType",
+                ...     group_by=[
+                ...         {"property": "tag"}
+                ...         {"property": "week"}
+                ...     ],
+                ...     order_by=[
+                ...         ComputeOrder({"property": "week"}, "ASC"),
+                ...         ComputeOrder({"property": "tag"}, "DESC")
+                ...     ],
+                ...     output={
+                ...         "myTag": {"property": "tag"}
+                ...         "myWeek": {"property": "week"}
+                ...         "myCount": {"count": {"function": {"property": "tag"}}}
+                ...     }
+                ... )
         """
         sub_computes_json = {"subComputes": sub_computes} if sub_computes is not None else {}
         from_feature_type_json = {"fromFeatureType": from_feature_type} if from_feature_type is not None else {}
         filter_json = {"filter": filter} if filter is not None else {}
+        group_by_json = {"groupBy": group_by} if group_by is not None else {}
+        order_by_json = (
+            {"orderBy": [{"expression": it.expression, "order": it.direction} for it in order_by]}
+            if order_by is not None
+            else {}
+        )
         output_json = {"output": output} if output is not None else {}
         binary_output_json = {"binaryOutput": binary_output} if binary_output is not None else {}
         res = self._post(
@@ -449,6 +493,8 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
                 **sub_computes_json,
                 **from_feature_type_json,
                 **filter_json,
+                **group_by_json,
+                **order_by_json,
                 **output_json,
                 **binary_output_json,
             },
@@ -459,3 +505,65 @@ class ExperimentalGeospatialAPI(GeospatialAPI):
         if content_type == "image/tiff":
             return res.content
         raise ValueError(f"unsupported content type ${content_type}")
+
+    def upsert_features(
+        self,
+        feature_type_external_id: str,
+        feature: Union[Feature, Sequence[Feature], FeatureList],
+        allow_crs_transformation: bool = False,
+        chunk_size: int = None,
+    ) -> Union[Feature, FeatureList]:
+        """`Upsert features`
+        <https://pr-1814.specs.preview.cogniteapp.com/v1.json.html#tag/Geospatial/operation/upsertFeatures>
+
+        Args:
+            feature_type_external_id: Feature type definition for the features to upsert.
+            feature: one feature or a list of features to upsert or a FeatureList object
+            allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
+                System defined in the feature type specification. When it is false, then requests with geometries in
+                Coordinate Reference System different from the ones defined in the feature type will result in
+                CogniteAPIError exception.
+            chunk_size: maximum number of items in a single request to the api
+
+        Returns:
+            Union[Feature, FeatureList]: Upserted features
+
+        Examples:
+
+            Upsert some features:
+
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> feature_types = [
+                ...     FeatureType(
+                ...         external_id="my_feature_type",
+                ...         properties={
+                ...             "location": {"type": "POINT", "srid": 4326},
+                ...             "temperature": {"type": "DOUBLE"}
+                ...         }
+                ...     )
+                ... ]
+                >>> res = c.geospatial.create_feature_types(feature_types)
+                >>> res = c.geospatial.upsert_features(
+                ...     feature_type_external_id="my_feature_type",
+                ...     feature=Feature(
+                ...         external_id="my_feature",
+                ...         location={"wkt": "POINT(1 1)"},
+                ...         temperature=12.4
+                ...     )
+                ... )
+        """
+        if chunk_size is not None and (chunk_size < 1 or chunk_size > self._CREATE_LIMIT):
+            raise ValueError(f"The chunk_size must be strictly positive and not exceed {self._CREATE_LIMIT}")
+        if isinstance(feature, FeatureList):
+            feature = list(feature)
+        resource_path = self._feature_resource_path(feature_type_external_id) + "/upsert"
+        extra_body_fields = {"allowCrsTransformation": "true"} if allow_crs_transformation else {}
+        return self._create_multiple(
+            list_cls=FeatureList,
+            resource_cls=Feature,
+            items=feature,
+            resource_path=resource_path,
+            extra_body_fields=extra_body_fields,
+            limit=chunk_size,
+        )
